ButtonTray = class(Turbine.UI.Control);

local ButtonImages = {
    CreateText = "Thurallor/Blackboard/Images/create_text.tga";
    CreateImage = "Thurallor/Blackboard/Images/create_image.tga";
    Thickness = "Thurallor/Blackboard/Images/select_thickness.tga";
    CreateGeometric = "Thurallor/Blackboard/Images/create_geometric.tga";
    CreateMap = "Thurallor/Blackboard/Images/create_map.tga";
    TextAlignment = "Thurallor/Blackboard/Images/select_alignment.tga";
}

function ButtonTray:Constructor()
    Turbine.UI.Control.Constructor(self);
    self.buttons = {};
    self.buttonOrder = {};
    self.showingButtons = {};
    self.globalButtonStates = {
        ForeColor = Turbine.UI.Color.White;
        BackColor = Turbine.UI.Color.Black;
        Thickness = 1;
        TextAlignment = Turbine.UI.ContentAlignment.MiddleCenter;
    };

    local button;

    self:_CreateButton("CreateText");
    self:_CreateButton("CreateImage");
    self:_CreateButton("CreateGeometric");

    button = self:_CreateButton("CreateMap");
    if (not _G.TerrainMap) then
        button:SetEnabled(false);
        button.tooltipText = "TerrainMap is not installed";
    end
    
    button = self:_CreateSelectorButton("Thickness", { 1, 2, 4 });
    
    button = self:_CreateSelectorButton("TextAlignment", { 
        Turbine.UI.ContentAlignment.TopLeft,
        Turbine.UI.ContentAlignment.TopCenter,
        Turbine.UI.ContentAlignment.TopRight,
        Turbine.UI.ContentAlignment.MiddleLeft,
        Turbine.UI.ContentAlignment.MiddleCenter,
        Turbine.UI.ContentAlignment.MiddleRight,
        Turbine.UI.ContentAlignment.BottomLeft,
        Turbine.UI.ContentAlignment.BottomCenter,
        Turbine.UI.ContentAlignment.BottomRight
    });

    button = self:_CreateButton("Colors");
    button.backSwatch = self:_AddColorSwatch(button, 20);
    button.foreSwatch = self:_AddColorSwatch(button, 20);
    button.foreSwatch:SetPosition(12, 12);

    button = self:_CreateButton("Color");
    button.swatch = self:_AddColorSwatch(button, 32);
end
    
function ButtonTray:_CreateButton(name)
    local button = Turbine.UI.Control();
    button.name = name;
    button:SetBackground(ButtonImages[name]);
    button:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
    button:SetSize(32, 32);
    button.tooltipText = "(Tooltip missing)";
    function button.SetEnabled(b, enabled)
        if (enabled) then
            button.MouseClick = function(b)
                self:_ButtonClicked(b.name)
            end
        else
            button.MouseClick = nil;
        end
    end
    function button.IsEnabled(b)
        return button.MouseClick;
    end
    button.MouseEnter = function(b)
        if (b:IsEnabled()) then
            b:SetBackColor(Turbine.UI.Color.Goldenrod);
        else
            b:SetBackColor(Turbine.UI.Color.Red);
        end
    end
    button.MouseLeave = function(b)
        b:SetBackColor(Turbine.UI.Color.White);
    end
    button:SetEnabled(true);
    self.buttons[name] = button;
    table.insert(self.buttonOrder, name);
    Thurallor.UI.Tooltip(function(b) return b.tooltipText; end):Attach(button);
    return button;
end

function ButtonTray:_CreateSelectorButton(name, values)
    local button = self:_CreateButton(name);
    button:SetBackground(nil);

    button.values = values;
    button.indexes = {};
    for n, v in ipairs(values) do
        button.indexes[v] = n;
    end
    
    local background = Turbine.UI.Control();
    background:SetBackground(ButtonImages[name]);
    background:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
    background:SetParent(button);
    background:SetSize(#values * 32, 32);
    background:SetMouseVisible(false);
    button.background = background;

    button.SetBackColor = function(b, color)
        b.background:SetBackColor(color);
    end
    button.SetValue = function(b, value)
        local n = b.indexes[value];
        b.background:SetLeft((n - 1) * -32);
    end
    button.GetValue = function(b)
        local left = b.background:GetLeft();
        return b.values[(left / -32) + 1];
    end
    button.IncrementValue = function(b)
        local value = b:GetValue();
        local n = b.indexes[value];
        n = n + 1;
        if (n > #values) then
            n = 1;
        end
        value = b.values[n];
        b:SetValue(value);
    end
end

function ButtonTray:_AddColorSwatch(button, size)
    local swatch = Turbine.UI.Control();
    swatch:SetParent(button);
    swatch:SetSize(size, size);
    swatch:SetBackColor(Turbine.UI.Color(1, 0.75, 0.75, 0.75));
    swatch.interior = Turbine.UI.Control();
    swatch.interior:SetParent(swatch);
    swatch.interior:SetSize(size - 2, size - 2);
    swatch.interior:SetPosition(1, 1);
    swatch.tooltipText = "(Tooltip missing)";
    swatch.interior.MouseClick = function(swi)
        self:_ButtonClicked(swi:GetParent());
    end
    swatch.interior.MouseEnter = function(swi)
        swi:GetParent():SetBackColor(Turbine.UI.Color.Goldenrod);
    end
    swatch.interior.MouseLeave = function(swi)
        swi:GetParent():SetBackColor(Turbine.UI.Color.White);
    end
    function swatch.SetColor(sw, color)
        sw.interior:SetBackColor(color);
    end
    function swatch.GetColor(sw)
        return sw.interior:GetBackColor();
    end
    Thurallor.UI.Tooltip(function(i) return i:GetParent().tooltipText; end):Attach(swatch.interior);
    return swatch;
end

function ButtonTray:SetSize(width, height)
    Turbine.UI.Control.SetSize(self, width, height);
    self:_Redraw();
end

function ButtonTray:SetColor(color)
    self.buttons.Color.swatch:SetColor(color);
end

function ButtonTray:GetColor()
    return self.buttons.Color.swatch:GetColor();
end

function ButtonTray:SetForeColor(color)
    self.buttons.Colors.foreSwatch:SetColor(color);
end

function ButtonTray:GetForeColor()
    return self.buttons.Colors.foreSwatch:GetColor();
end

function ButtonTray:SetBackColor(color)
    self.buttons.Colors.backSwatch:SetColor(color);
end

function ButtonTray:GetBackColor()
    return self.buttons.Colors.backSwatch:GetColor();
end

function ButtonTray:SetThickness(thickness)
    return self.buttons.Thickness:SetValue(thickness);
end

function ButtonTray:GetThickness()
    return self.buttons.Thickness:GetValue();
end

function ButtonTray:SetTextAlignment(alignment)
    return self.buttons.TextAlignment:SetValue(alignment);
end

function ButtonTray:GetTextAlignment()
    return self.buttons.TextAlignment:GetValue();
end

function ButtonTray:_ButtonClicked(button)
    if ((button == self.buttons.Color.swatch) or (button == self.buttons.Colors.foreSwatch) or (button == self.buttons.Colors.backSwatch)) then
        self:_PickColor(button);
    elseif (button == "Thickness") then
        button = self.buttons.Thickness;
        button:IncrementValue();
        DoCallbacks(self, "ThicknessChanged");
    elseif (button == "TextAlignment") then
        button = self.buttons.TextAlignment;
        button:IncrementValue();
        DoCallbacks(self, "TextAlignmentChanged");
    end
    DoCallbacks(self, "ButtonClicked", button);
end

function ButtonTray:_PickColor(swatch)
    if (self.colorPicker) then
        self.colorPicker:Close();
    end
    
    local colorPicker = Thurallor.UI.ColorPicker(swatch:GetColor());
    local function colorChanged()
        self.currentSwatch:SetColor(self.colorPicker:GetColor());
        DoCallbacks(self, "ColorChanged");
    end
    AddCallback(colorPicker, "ColorChanged", colorChanged);
    AddCallback(colorPicker, "Accepted", {colorChanged, function() self.colorPicker = nil; end});
    AddCallback(colorPicker, "Canceled", {colorChanged, function() self.colorPicker = nil; end});
    
    self.colorPicker = colorPicker;
    self.currentSwatch = swatch;
end

function ButtonTray:SetContext(shapes)
--Puts("ButtonTray:SetContext " .. Serialize({context, shapeTypes}));
    self.shape = nil;
    self.context = context;
    self.showingButtons = {};
    if (shapes == nil) then
        return self:_Redraw();
    end

    local function showButton(name, tooltipText)
        local button = self.buttons[name];
        if (button) then
            self.showingButtons[name] = true;
            if (button:IsEnabled()) then
                self.buttons[name].tooltipText = tooltipText;
            end
        end
    end
    
    local shapeClass = next(shapes);
    self.showingButtons = {};
    if (next(shapes, shapeClass)) then
        -- Multiple types of shape selected.  Hide all buttons.
        -- To do: Handle selection of multiple shape types: show only the properties that apply to all of them.
    else
        if (#shapes[shapeClass] > 1) then
            -- Multiple shapes of this type selected.  Hide all buttons.
            -- To do: Handle selection of multiple shapes of the same type
        else
            self.shape = shapes[shapeClass][1];
            if (shapeClass == Canvas) then
                showButton("CreateText", "Create a new text shape");
                showButton("CreateImage", "Create a new image shape");
                showButton("CreateGeometric", "Create a new geometric shape");
                showButton("CreateMap", "Create a new radar map shape");
                showButton("TextAlignment", "Set the default alignment for new text shapes");
                showButton("Thickness", "Set the default line thickness for new geometric shapes");
                showButton("Colors", "Set the default colors for new shapes");
                self.buttons.Colors.foreSwatch.tooltipText = "Set the default foreground color for new shapes";
                self.buttons.Colors.backSwatch.tooltipText = "Set the background color for the window and the default for new shapes";

                -- Attach button states to shape properties
                self:SetBackColor(self.shape:GetBackColor());
                self.ColorChanged = function()
                    self.shape:SetBackColor(self:GetBackColor());
                end
                
                -- For foreground color and thickness buttons, we have two meanings depending on context: default or shape-specific.  If canvas is selected, we are editing the defaults for new shapes.
                self:SetForeColor(self.globalButtonStates.ForeColor);
                self:SetThickness(self.globalButtonStates.Thickness);
                self:SetTextAlignment(self.globalButtonStates.TextAlignment);
                self.ColorChanged = { self.ColorChanged, function()
                    self.globalButtonStates.ForeColor = self:GetForeColor();
                end };
                self.ThicknessChanged = function()
                    self.globalButtonStates.Thickness = self:GetThickness();
                end
                self.TextAlignmentChanged = function()
                    self.globalButtonStates.TextAlignment = self:GetTextAlignment();
                end
            elseif (shapeClass == TextShape) then
                showButton("TextAlignment", "Set the alignment for the text in the selected shape");
                showButton("Color", "Change the text color for the selected shape");
                self.buttons.Color.swatch.tooltipText = self.buttons.Color.tooltipText;

                -- Attach button states to shape properties
                self:SetColor(self.shape:GetColor());
                self:SetTextAlignment(self.shape:GetTextAlignment());
                self.ColorChanged = function()
                    self.shape:SetColor(self:GetColor());
                end
                self.TextAlignmentChanged = function()
                    self.shape:SetTextAlignment(self:GetTextAlignment());
                end
            elseif (shapeClass == RectangleShape) then
                showButton("Thickness", "Change the outline thickness for the selected shape");
                showButton("Colors", "Change the colors for the selected shape");
                self.buttons.Colors.foreSwatch.tooltipText = "Change the outline color for the selected shape";
                self.buttons.Colors.backSwatch.tooltipText = "Change the interior color for the selected shape";
                
                -- Attach button states to shape properties
                self:SetThickness(self.shape:GetOutlineSize());
                self.ThicknessChanged = function()
                    self.shape:SetOutlineSize(self:GetThickness());
                end
                self:SetForeColor(self.shape:GetForeColor());
                self:SetBackColor(self.shape:GetBackColor());
                self.ColorChanged = function()
                    self.shape:SetForeColor(self:GetForeColor());
                    self.shape:SetBackColor(self:GetBackColor());
                end
            end
        end
    end

    self:_Redraw();
end

function ButtonTray:_Redraw()
    self:GetControls():Clear();
    local spacing = 4;
    local width, height = self:GetSize();

    local container = Turbine.UI.Control();
    container:SetParent(self);
    container:SetSize(width, height);

    local button, swatch;
    local top = 0;
    for _, name in ipairs(self.buttonOrder) do
        if (self.showingButtons[name]) then
            local button = self.buttons[name];
            button:SetParent(container);
            button:SetPosition(0, top);
            top = top + button:GetHeight() + spacing;
        end
    end
    
    -- Center the container vertically
    top = math.max(0, top - spacing);
    container:SetHeight(top);
    container:SetTop(math.floor(0.5 + (height - top) / 2));
end
